<?php

/*------------------------------------------------------------------------
# com_invoices - Invoice Manager for Joomla
# ------------------------------------------------------------------------
# author                Germinal Camps
# copyright             Copyright (C) 2012 - 2016 JoomlaThat.com. All Rights Reserved.
# @license              http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
# Websites:             http://www.joomlathat.com
# Technical Support:    Forum - http://www.joomlathat.com/support
-------------------------------------------------------------------------*/

//no direct access
defined('_JEXEC') or die('Restricted access.');

jimport('joomla.plugin.plugin');

class plgInvoicesPayment_stripe extends JPlugin
{

  var $_payment_type = 'payment_stripe';
  var $contact = null;
  var $contactparams = null;

  function __construct(& $subject, $config) {
    parent::__construct($subject, $config);
    $this->loadLanguage( '', JPATH_ADMINISTRATOR );
  }

  function onRenderPaymentOptions( $row, $user )
  {
    $compparams = JComponentHelper::getParams( 'com_invoices' );
    
    require_once('config.php');

    $vars = new JObject();

    $db = JFactory::getDBO();

    if(!$row->currency_id) $row->currency_id = $compparams->get('currency_id', 1);
    $query = "SELECT * FROM #__invoices_currencies
    WHERE id = ".(int)$row->currency_id;
    $db->setQuery($query);
    $currency = $db->loadObject();

    $vars->note = $this->params->get( 'description' );
    $vars->row = $row;
    $vars->currency = $currency->currency_code;
    $vars->user = $user;

    $row->currency = $currency->currency_code;

    $vars->action_url = JURI::base().'index.php?option=com_invoices&task=process_payment&ptype='.$this->_payment_type.'&paction=process&tmpl=component&item_number='.$row->id;

    $row->action_url = $vars->action_url;

    $vars->stripe_session_id = $this->createStripeSession($row);

    //comes from config.php file
    $vars->stripe_publishable_key = $stripe['publishable_key'];

    $html = $this->_getLayout('form', $vars);

    $text = array();
    $text[] = $html;
    $text[] = $this->params->get( 'title', 'Stripe' );

    return $text;
  }

  function createStripeSession($row){
    //test card: 4000000000003220
    //https://stripe.com/docs/payments/3d-secure
    //https://stripe.com/docs/payments/checkout/migration#api-products
    $item_name = 'Payment ' . $row->id . ' (Invoice '.$row->invoice_num.')';

    $this->loadContactFromPaymentId($row->id);

    $customer_id = $this->contactparams->get('stripe_customer_id',null);

    //https://stripe.com/docs/api/checkout/sessions/object
    $session = \Stripe\Checkout\Session::create([
      'payment_method_types' => ['card'],
      'client_reference_id' => $row->id,
      'billing_address_collection' => $this->params->get('billing') ? 'required' : 'auto',
      'customer' => $customer_id,
      'line_items' => [[
        'name' => $item_name,
        'description' => $row->payment_description ?: $item_name,
        'amount' => $row->payment_amount * 100,
        'currency' => $row->currency,
        'quantity' => 1
        ]],
        'success_url' => $row->action_url,
        'cancel_url' => $row->action_url,
      ]);

      return $session->id;
    }

    function onProcessPayment($row, $user)
    {
      require_once('config.php');

      $ptype 		= JRequest::getVar( 'ptype' );

      if ($ptype == $this->_payment_type)
      {
        $paction 	= JRequest::getVar( 'paction' );
        $html = "";

        switch ($paction) {
          case "display_message":

          break;
          case "process":

          break;
          case "cancel":

          break;
          case "webhook":
          $this->webhook();
          break;
          default:

          break;
        }

        return $html;
      }
      return;
    }

    function webhook(){

      //https://stripe.com/docs/webhooks/signatures
      // You can find your endpoint's secret in your webhook settings
      $endpoint_secret = $this->params->get( 'webhook_secret', '' );

      //https://stripe.com/docs/webhooks/setup
      $payload = @file_get_contents('php://input');
      $sig_header = $_SERVER['HTTP_STRIPE_SIGNATURE'];
      $event = null;

      //verify the weebhook origin! (only if we provided it)
      if($endpoint_secret){
        try {
            $event = \Stripe\Webhook::constructEvent(
                $payload, $sig_header, $endpoint_secret
            );
        } catch(\UnexpectedValueException $e) {
            // Invalid payload
            http_response_code(400);
            exit();
        } catch(\Stripe\Error\SignatureVerification $e) {
          $this->_logData($e);
            // Invalid signature
            http_response_code(400);
            exit();
        }

      }

      try {
        $event = \Stripe\Event::constructFrom(
          json_decode($payload, true)
        );
      } catch(\UnexpectedValueException $e) {
        // Invalid payload
        $this->_logData($e);
        http_response_code(400);
        exit();
      }

      // Handle the event
      switch ($event->type) {
        case 'payment_intent.succeeded':
        // $paymentIntent = $event->data->object; // contains a \Stripe\PaymentIntent
        // $this->handlePaymentIntentSucceeded($paymentIntent);
        break;
        case 'payment_method.attached':
        // $paymentMethod = $event->data->object; // contains a \Stripe\PaymentMethod
        // $this->handlePaymentMethodAttached($paymentMethod);
        break;
        case 'checkout.session.completed':
        $session = $event->data->object;
        $this->handleCheckoutSession($session);
        break;
        // ... handle other event types
        default:
        // Unexpected event type
        http_response_code(400);
        exit();
      }

      http_response_code(200);

    }

    protected function handlePaymentIntentSucceeded($paymentIntent)
    {
      $this->_logData($paymentIntent);
      //https://stripe.com/docs/payments/checkout/fulfillment#webhooks
    }

    protected function handleCheckoutSession($session)
    {
      $this->_logData($session);
      $this->_process($session);
      //https://stripe.com/docs/payments/checkout/fulfillment#webhooks
    }

    protected function handlePaymentMethodAttached($paymentMethod)
    {
      $this->_logData($paymentMethod);
    }

    private function _process($session)
    {

      //https://stripe.com/docs/payments/checkout/fulfillment

      $app = JFactory::getApplication();

      $params = JComponentHelper::getParams( 'com_invoices' );

      $db = JFactory::getDBO();

      $payment_id = (int)$session->client_reference_id;

      $payment = InvoicesHelper::getPaymentData($payment_id);

      $query = "SELECT currency_id FROM #__invoices_invoices
      WHERE id = ".(int)$payment->invoice_id;
      $db->setQuery($query);
      $currency_id = $db->loadResult();

      if(!$currency_id) $currency_id = $params->get('currency_id', 1);

      $query = "SELECT * FROM #__invoices_currencies
      WHERE id = ".(int)$currency_id;
      $db->setQuery($query);
      $currency = $db->loadObject();

      $this->loadContactFromPaymentId($payment_id);

      $customer_id = $this->contactparams->get('stripe_customer_id','');

      if(!$customer_id){
        $customer_id = $session->customer;
      }

      $this->saveCustomerId($customer_id);

      //charge was correct, let's save it in the DB
      $model = $this->getPaymentsModel();
      $payment->payment_status = 1; //paid
      $payment->payment_type = $this->_payment_type;
      $payment->payment_datetime = date("Y-m-d H:i:s");

      $model->store(get_object_vars($payment));

      $app->enqueueMessage('Payment successful!');

      // try {
      //     $charge = \Stripe\Charge::create(array(
      //         'customer'  => $customer_id,
      //         'amount'    => $payment->payment_amount * 100, // *100 because stripe amount only accepts cents.
      //         'currency'  => $currency->currency_code
      //     ));
      //
      //     //save the Customer ID to the DB
      //     $this->saveCustomerId($customer_id);
      //
      //     if ($charge->__get('paid')) {
      //
      //         $model = $this->getPaymentsModel();
      //         $payment->payment_status = 1; //paid
      //         $payment->payment_type = $this->_payment_type;
      //         $payment->payment_datetime = date("Y-m-d H:i:s");
      //
      //         $model->store(get_object_vars($payment));
      //
      //     }
      //
      //     $app->enqueueMessage($charge->__get('status'));
      //
      // } catch(Exception $e) {
      //     $msg = $e->getMessage();
      //     $app->enqueueMessage($msg);
      // }
    }

    private function loadContactFromPaymentId($payment_id){
      $db = JFactory::getDBO();
      $query = ' SELECT co.* FROM #__invoices_contacts AS co '
      .' LEFT JOIN #__invoices_invoices AS i ON i.user_id = co.id '
      .' LEFT JOIN #__invoices_payments AS pa ON pa.invoice_id = i.id '
      .' WHERE pa.id = '.(int)$payment_id
      ;
      $db->setQuery($query);

      $this->contact = $db->loadObject();
      $this->contactparams = new JRegistry($this->contact->params);
    }

    private function saveCustomerId($customer_id){
      $model = $this->getContactModel();

      $this->contactparams->set('stripe_customer_id',$customer_id);
      $params = $this->contactparams->toString();

      $contact = array( 'id'      => $this->contact->id,
      'vincular_cliente_checkbox' => 1,
      'params'  => $params);

      $model->store($contact);
    }

    private function _getLayout($layout, $vars = false, $plugin = '', $group = 'invoices')
    {
      if ( ! $plugin) {
        $plugin = $this->_payment_type;
      }

      ob_start();
      $layout = $this->_getLayoutPath( $plugin, $group, $layout );
      include($layout);
      $html = ob_get_contents();
      ob_end_clean();

      return $html;
    }

    private function _getLayoutPath($plugin, $group, $layout = 'default')
    {
      $app = JFactory::getApplication();

      // get the template and default paths for the layout
      $templatePath = JPATH_BASE.DS.'templates'.DS.$app->getTemplate().DS.'html'.DS.'plugins'.DS.$group.DS.$plugin.DS.$plugin.DS.$layout.'.php';
      $defaultPath = JPATH_BASE.DS.'plugins'.DS.$group.DS.$plugin.DS.$plugin.DS.'tmpl'.DS.$layout.'.php';

      // if the site template has a layout override, use it
      jimport('joomla.filesystem.file');
      if (JFile::exists( $templatePath ))
      {
        return $templatePath;
      }
      else
      {
        return $defaultPath;
      }
    }

    private function getPaymentsModel()
    {
      if (!class_exists( 'PaymentsModelPayment' ))
      {
        // Build the path to the model based upon a supplied base path
        $path = JPATH_SITE.DS.'administrator'.DS.'components'.DS.'com_invoices'.DS.'models'.DS.'payment.php';
        $false = false;

        // If the model file exists include it and try to instantiate the object
        if (file_exists( $path )) {
          require_once( $path );
          if (!class_exists( 'PaymentsModelPayment' )) {
            JError::raiseWarning( 0, 'Model class PaymentsModelPayment not found in file.' );
            return $false;
          }
        } else {
          JError::raiseWarning( 0, 'Model PaymentsModelPayment not supported. File not found.' );
          return $false;
        }
      }

      $model = new PaymentsModelPayment();
      return $model;
    }

    private function getContactModel()
    {
      if (!class_exists( 'ContactsModelContact' ))
      {
        // Build the path to the model based upon a supplied base path
        $path = JPATH_SITE.DS.'administrator'.DS.'components'.DS.'com_invoices'.DS.'models'.DS.'contact.php';
        $false = false;

        // If the model file exists include it and try to instantiate the object
        if (file_exists( $path )) {
          require_once( $path );
          if (!class_exists( 'ContactsModelContact' )) {
            JError::raiseWarning( 0, 'Model class ContactsModelContact not found in file.' );
            return $false;
          }
        } else {
          JError::raiseWarning( 0, 'Model ContactsModelContact not supported. File not found.' );
          return $false;
        }
      }

      $model = new ContactsModelContact();
      return $model;
    }

    function _logData($data)
  	{
  		$f = fopen(JPATH_CACHE . '/im-stripe.txt', 'a');
  		fwrite($f, "\n" . date('F j, Y, g:i a') . "\n");
  		fwrite($f, print_r($data, true));
  		fclose($f);
  	}

  }
